/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * Copyright (C) 2010 Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Jan Görig <jgorig@redhat.com>
 */

#include <stdlib.h>
#include <netinet/in.h>
#include "nslist.h"

/* Initialize list */
void nslist_init(nslist_t *list)
{
	list->head = NULL;
	list->tail = NULL;
	list->count = 0;
}

/* Free list */
void nslist_free(nslist_t *list)
{
	nslist_item_p item;

	while (list->head != NULL) {
		item = list->head;
		list->head = list->head->next;
		free(item);
	}

	list->tail = NULL;
}

/* Find lists item */
nslist_item_p nslist_find(nslist_t *list, in_addr_t address)
{
	nslist_item_p item;

	item = list->head;

	while (item != NULL) {
		if (item->address == address)
			return item;

		item = item->next;
	}

	return NULL;
}

/* Adds a new list item if not present in list */
bool nslist_add(nslist_t *list, in_addr_t address)
{
	nslist_item_p item;

	if (nslist_find(list, address) != NULL)
		return true;

	item = malloc(sizeof(struct nslist_item));

	if (item == NULL)
		return false;

	list->count++;
	item->address = address;
	item->next = NULL;

	if (list->head == NULL) {
		list->head = item;
		list->tail = item;
	} else {
		list->tail->next = item;
		list->tail = item;
	}

	return true;
}

/* Returns true for identical lists */
bool nslist_compare(nslist_t *list1, nslist_t *list2)
{
	nslist_item_p item1, item2;

	item1 = list1->head;
	item2 = list2->head;

	while (item1 != NULL && item2 != NULL) {
		if (item1->address != item2->address)
			return false;

		item1 = item1->next;
		item2 = item2->next;
	}

	return item1 == item2;
}

/* Frees target and move source to target */
void nslist_move(nslist_t *target, nslist_t *source)
{
	nslist_free(target);
	*target = *source;
	nslist_init(source);
}

/* Returns list converted to array  */
in_addr_t* nslist_serialize(nslist_t *list)
{
	int i;
	in_addr_t *addrs;
	nslist_item_p item;

	addrs = malloc((list->count + 1) * sizeof(in_addr_t));

	if (addrs == NULL)
		return NULL;

	item = list->head;

	i = 0;
	while (item != NULL) {
		addrs[i] = item->address;
		item = item->next;

		i++;
	}

	addrs[i] = 0;

	return addrs;
}
